/* **********************************************************
 * Copyright 2001 VMware, Inc.  All rights reserved. -- VMware Confidential
 * **********************************************************/

/*
 * pshare_ext.h --
 *
 *	VMKernel/VMMon <-> VMM transparent page sharing info.
 */

#ifndef _PSHARE_EXT_H
#define _PSHARE_EXT_H

#define INCLUDE_ALLOW_USERLEVEL
#define INCLUDE_ALLOW_VMM
#define INCLUDE_ALLOW_VMKERNEL
#define INCLUDE_ALLOW_MODULE
#define INCLUDE_ALLOW_VMCORE
#include "includeCheck.h"

/*
 * constants
 */

#ifdef VMX86_SERVER
#define PSHARE_BATCH_PAGES_MAX          (64)
#else
#define PSHARE_BATCH_PAGES_MAX          (32)
#endif

#define PSHARE_P2M_BUFFER_MPNS_MAX      (8)
#define PSHARE_P2M_BUFFER_MPNS_DEFAULT  (2)
#define	PSHARE_P2M_UPDATES_MAX		(64)
#define	PSHARE_P2M_UPDATES_OVERFLOW	(-1)
#define	PSHARE_HINT_UPDATES_MAX		(PSHARE_BATCH_PAGES_MAX)
#define PSHARE_P2M_BUFFER_SLOTS_PER_MPN ((PAGE_SIZE/sizeof(PShare_P2MUpdate)))

#define PSHARE_MAX_COW_CHECK_PAGES      (16)

/*
 * types
 */

typedef struct PShare_P2MUpdate {
   PageNum page;
   MPN     mpn;
   Bool    isCow;
} PShare_P2MUpdate;

typedef enum {
   PSHARE_HINT_NONE,
   PSHARE_HINT_STALE,
   PSHARE_HINT_MATCH
} PShare_HintStatus;

typedef struct PShare_HintUpdate {
   PageNum           page;
   PShare_HintStatus status;
} PShare_HintUpdate;

typedef struct {
   uint32 nScan;	// pages scanned
   uint32 nAttempt;	// page sharing attempts
   uint32 nCOW;		// pages marked COW
   uint32 nHint;	// pages marked COW hint
   uint32 nShare;	// pages shared
   uint32 nCopy;	// pages copied

   uint32 nCheck;	// pages checked
   uint32 nCheckBad;	// page check failures
   uint32 nCheckBadKey;	// hash key check mismatches
   uint32 nCheckBadMPN;	// MPN check mismatches
   uint32 nCheckBadCOW;	// COW check mismatches
} PShare_MonitorStats;

typedef struct PShare_COWCheckInfo {
   PageNum pageNum;    // Page num to check
   MPN     vmmMPN;     // mpn for this page in monitor
   MPN     hostMPN;    // mpn for this page in the host
   Bool    vmmCOW;     // cow state of this page in monitor
   Bool    hostCOW;    // cow state of this page in the host
   Bool    keyOK;
   Bool    checkOK;
} PShare_COWCheckInfo;

#endif
